@@ -11,7 +11,7 @@
 
    Regular Expression Matcher working on lists of XmlTrees
 
-   It's intendet to import this module with an explicit
+   It's intended to import this module with an explicit
    import declaration for not spoiling the namespace
    with these somewhat special arrows
 
@@ -24,6 +24,7 @@ module Text.XML.HXT.Arrow.XmlRegex
     , mkZero
     , mkUnit
     , mkPrim
+    , mkPrim'
     , mkPrimA
     , mkDot
     , mkStar
@@ -34,6 +35,9 @@ module Text.XML.HXT.Arrow.XmlRegex
     , mkRep
     , mkRng
     , mkOpt
+    , mkPerm
+    , mkPerms
+    , mkMerge
     , nullable
     , delta
     , matchXmlRegex
@@ -50,6 +54,7 @@ import Control.Arrow.ListArrows
 import Data.Maybe
 
 import Text.XML.HXT.DOM.Interface
+import Text.XML.HXT.DOM.ShowXml  ( xshow )
 
 -- ------------------------------------------------------------
 -- the exported regex arrows
@@ -88,13 +93,15 @@ scanRegexA re ts        = ts >>. (fromMaybe [] . scanXmlRegex re)
 
 data XmlRegex   = Zero String
                 | Unit
-                | Sym (XmlTree -> Bool)
+                | Sym  (XmlTree -> Bool) String    -- optional external repr. of predicate
                 | Dot
-                | Star XmlRegex
-                | Alt XmlRegex XmlRegex
-                | Seq XmlRegex XmlRegex
-                | Rep Int XmlRegex              -- 1 or more repetitions
-                | Rng Int Int XmlRegex  -- n..m repetitions
+                | Star  XmlRegex
+                | Alt   XmlRegex XmlRegex
+                | Seq   XmlRegex XmlRegex
+                | Rep   Int      XmlRegex          -- 1 or more repetitions
+                | Rng   Int Int  XmlRegex          -- n..m repetitions
+                | Perm  XmlRegex XmlRegex
+                | Merge XmlRegex XmlRegex
 
 -- ------------------------------------------------------------
 
@@ -106,7 +113,7 @@ class Inv a where
 instance Inv XmlRegex where
     inv (Zero _)        = True
     inv Unit            = True
-    inv (Sym p)         = p holds for some XmlTrees
+    inv (Sym p _)       = p holds for some XmlTrees
     inv Dot             = True
     inv (Star e)        = inv e
     inv (Alt e1 e2)     = inv e1 &&
@@ -116,6 +123,8 @@ instance Inv XmlRegex where
     inv (Rep i e)       = i > 0 && inv e
     inv (Rng i j e)     = (i < j || (i == j && i > 1)) &&
                           inv e
+    inv (Perm e1 e2)    = inv e1 &&
+                          inv e2
 -}
 -- ------------------------------------------------------------
 --
@@ -128,7 +137,10 @@ mkUnit          :: XmlRegex
 mkUnit          = Unit
 
 mkPrim          :: (XmlTree -> Bool) -> XmlRegex
-mkPrim          = Sym
+mkPrim p        = Sym p ""
+
+mkPrim'         :: (XmlTree -> Bool) -> String -> XmlRegex
+mkPrim'         = Sym
 
 mkPrimA         :: LA XmlTree XmlTree -> XmlRegex
 mkPrimA a       = mkPrim (not . null . runLA a)
@@ -155,9 +167,14 @@ mkAlt e1            (Zero _)            = e1                            -- e1 u
 mkAlt (Zero _)      e2                  = e2                            -- {} u e2 = e2
 mkAlt e1@(Star Dot) _e2                 = e1                            -- A* u e1 = A*
 mkAlt _e1           e2@(Star Dot)       = e2                            -- e1 u A* = A*
-mkAlt (Sym p1)      (Sym p2)            = mkPrim $ \ x -> p1 x || p2 x  -- melting of predicates
-mkAlt e1            e2@(Sym _)          = mkAlt e2 e1                   -- symmetry: predicates always first
-mkAlt e1@(Sym _)    (Alt e2@(Sym _) e3) = mkAlt (mkAlt e1 e2) e3        -- prepare melting of predicates
+mkAlt (Sym p1 e1)   (Sym p2 e2)         = mkPrim' (\ x -> p1 x || p2 x)  (e e1 e2) -- melting of predicates
+                                          where
+                                            e "" x2 = x2
+                                            e x1 "" = x1
+                                            e x1 x2 = x1 ++ "|" ++ x2
+mkAlt e1            e2@(Sym _ _)        = mkAlt e2 e1                   -- symmetry: predicates always first
+mkAlt e1@(Sym _ _)  (Alt e2@(Sym _ _) e3)
+                                        = mkAlt (mkAlt e1 e2) e3        -- prepare melting of predicates
 mkAlt (Alt e1 e2)   e3                  = mkAlt e1 (mkAlt e2 e3)        -- associativity
 mkAlt e1 e2                             = Alt e1 e2
 
@@ -195,12 +212,30 @@ mkRng lb ub e                   = Rng lb ub e
 mkOpt   :: XmlRegex -> XmlRegex
 mkOpt   = mkRng 0 1
 
+mkPerm                           :: XmlRegex -> XmlRegex -> XmlRegex
+mkPerm e1@(Zero _) _             = e1
+mkPerm _           e2@(Zero _)   = e2
+mkPerm Unit        e2            = e2
+mkPerm e1          Unit          = e1
+mkPerm e1          e2            = Perm e1 e2
+
+mkPerms                          :: [XmlRegex] -> XmlRegex
+mkPerms                          = foldr mkPerm mkUnit
+
+mkMerge                          :: XmlRegex -> XmlRegex -> XmlRegex
+mkMerge e1@(Zero _) _            = e1
+mkMerge _           e2@(Zero _)  = e2
+mkMerge Unit        e2           = e2
+mkMerge e1          Unit         = e1
+mkMerge e1          e2           = Merge e1 e2
+
 -- ------------------------------------------------------------
 
 instance Show XmlRegex where
     show (Zero s)       = "{err:" ++ s ++ "}"
     show Unit           = "()"
-    show (Sym _p)       = "{single tree pred}"
+    show (Sym _p "")    = "<pred>"
+    show (Sym _p r )    = r
     show Dot            = "."
     show (Star e)       = "(" ++ show e ++ ")*"
     show (Alt e1 e2)    = "(" ++ show e1 ++ "|" ++ show e2 ++ ")"
@@ -209,13 +244,28 @@ instance Show XmlRegex where
     show (Rep i e)      = "(" ++ show e ++ "){" ++ show i ++ ",}"
     show (Rng 0 1 e)    = "(" ++ show e ++ ")?"
     show (Rng i j e)    = "(" ++ show e ++ "){" ++ show i ++ "," ++ show j ++ "}"
+    show (Perm e1 e2)   = "(" ++ show e1 ++ show e2 ++ "|" ++ show e2 ++ show e1 ++ ")"
+    show (Merge e1 e2)  = "(" ++ show e1 ++ "&" ++ show e2 ++ ")"
+
+-- ------------------------------------------------------------
+
+unexpected 		:: XmlTree -> String -> String
+unexpected t e		= emsg e ++ (cut 80 . xshow) [t]
+    where
+      emsg ""           = "unexpected: "
+      emsg s            = "expected: " ++ s ++ ", but got: "
+      cut n s
+          | null rest   = s'
+          | otherwise   = s' ++ "..."
+          where
+            (s', rest)  = splitAt n s
 
 -- ------------------------------------------------------------
 
 nullable        :: XmlRegex -> Bool
 nullable (Zero _)       = False
 nullable Unit           = True
-nullable (Sym _p)       = False         -- assumption: p holds for at least one tree
+nullable (Sym _p _)     = False         -- assumption: p holds for at least one tree
 nullable Dot            = False
 nullable (Star _)       = True
 nullable (Alt e1 e2)    = nullable e1 ||
@@ -225,25 +275,34 @@ nullable (Seq e1 e2)    = nullable e1 &&
 nullable (Rep _i e)     = nullable e
 nullable (Rng i _ e)    = i == 0 ||
                           nullable e
+nullable (Perm e1 e2)   = nullable e1 &&
+                          nullable e2
+nullable (Merge e1 e2)  = nullable e1 &&
+                          nullable e2
 
 -- ------------------------------------------------------------
 
 delta   :: XmlRegex -> XmlTree -> XmlRegex
-delta e@(Zero _)  _     = e
-delta Unit        c     = mkZero $
-                          "unexpected char " ++ show c
-delta (Sym p)     c
+delta e@(Zero _)   _    = e
+delta Unit         c    = mkZero $ unexpected c ""
+delta (Sym p e)    c
     | p c               = mkUnit
-    | otherwise         = mkZero $
-                          "unexpected tree " ++ show c
-delta Dot         _     = mkUnit
-delta e@(Star e1) c     = mkSeq (delta e1 c) e
-delta (Alt e1 e2) c     = mkAlt (delta e1 c) (delta e2 c)
-delta (Seq e1 e2) c
+    | otherwise         = mkZero $ unexpected c e
+delta Dot          _    = mkUnit
+delta e@(Star e1)  c    = mkSeq (delta e1 c) e
+delta (Alt e1 e2)  c    = mkAlt (delta e1 c) (delta e2 c)
+delta (Seq e1 e2)  c
     | nullable e1       = mkAlt (mkSeq (delta e1 c) e2) (delta e2 c)
     | otherwise         = mkSeq (delta e1 c) e2
-delta (Rep i e)   c     = mkSeq (delta e c) (mkRep (i-1) e)
-delta (Rng i j e) c     = mkSeq (delta e c) (mkRng ((i-1) `max` 0) (j-1) e)
+delta (Rep i e)    c    = mkSeq (delta e c) (mkRep (i-1) e)
+delta (Rng i j e)  c    = mkSeq (delta e c) (mkRng ((i-1) `max` 0) (j-1) e)
+delta (Perm e1 e2) c    = case e1' of
+                            (Zero _) -> mkPerm e1 (delta e2 c)
+                            _        -> mkPerm e1' e2
+                          where
+                          e1' = delta e1 c
+delta (Merge e1 e2) c   = mkAlt (mkMerge (delta e1 c) e2)
+                                (mkMerge e1 (delta e2 c))
 
 -- ------------------------------------------------------------
 
